﻿#region Copyright © convivo GmbH
// ------------------------------------------------------------------------------------------------------------------------
// <copyright file="BaseGridViewController.cs.cs" company="convivo GmbH">
//    Copyright © 2013 convivo GmbH
// </copyright>
// <creator name="convivo GmbH"/>
// ------------------------------------------------------------------------------------------------------------------------
#endregion

namespace Fruitmonitoring.Controllers.General
{
    #region usings
    using System;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;

    using Fruitmonitoring.Controllers.Interfaces;
    using Fruitmonitoring.Helper.Utility;
    using Fruitmonitoring.Infrastructure.ActionAttributes;
    using Fruitmonitoring.Infrastructure.Security;

    using Languages;

    #endregion

    /// <summary>
    /// The base grid view controller.
    /// </summary>
    public abstract class BaseGridViewController : BaseController
    {
        #region Constructors and Destructors

        #region constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="BaseGridViewController"/> class.
        /// constructor provides important informations to the view.
        /// </summary>
        /// <param name="controllerName">
        /// Name of the controller, which should handle the incomming gridview requests.
        /// </param>
        /// <param name="gridViewId">
        /// The unique gridview retialerCompanyId.
        /// </param>
        /// <param name="isAdmin">
        /// Set admin flag that can be used in the view.
        /// </param>
        /// <param name="isSuperUser">
        /// Set super user flag that can be used in the view.
        /// </param>
        public BaseGridViewController(string controllerName, string gridViewId, bool isAdmin, bool isSuperUser)
        {
        }
        #endregion

        #endregion

        #region Methods

        #region overrides
        /// <summary>
        /// The on action executed.
        /// </summary>
        /// <param name="filterContext">
        /// The filter context.
        /// </param>
        protected override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            // provide users company id
            if (ViewData.ContainsKey("CompanyId"))
            {
                ViewData.Remove("CompanyId");
            }

            ViewData.Add("CompanyId", User.GetCompanyGuid());

            base.OnActionExecuted(filterContext);
        }
        #endregion

        #endregion

        #region variables
        /// <summary>
        /// The grid view id.
        /// </summary>
        private readonly string gridViewId;

        // gridview helper object
        /// <summary>
        /// The _grid view helper.
        /// </summary>
        public GridViewHelper _gridViewHelper;

        /// <summary>
        /// Gets the grid view helper.
        /// </summary>
        public GridViewHelper GridViewHelper
        {
            get
            {
                if (_gridViewHelper == null)
                {
                    _gridViewHelper = new GridViewHelper();
                }

                return _gridViewHelper;
            }
        }

        // gridview filter helper object
        /// <summary>
        /// The _grid view filter helper.
        /// </summary>
        public GridViewFilterHelper _gridViewFilterHelper;

        /// <summary>
        /// Gets the grid view filter helper.
        /// </summary>
        public GridViewFilterHelper GridViewFilterHelper
        {
            get
            {
                if (_gridViewFilterHelper == null)
                {
                    _gridViewFilterHelper = new GridViewFilterHelper();
                }

                return _gridViewFilterHelper;
            }
        }
        #endregion

        #region abstract members
        /// <summary>
        /// function for rendering the gridview partial template
        /// The grid view list view.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public abstract ActionResult GridViewListView();

        /// <summary>
        /// Function for rendering the gridview.
        /// This function do returns no model for the gridview. 
        /// The Gridview stays empty and can filled with data client side (e.g. ajax requests).
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        public abstract ActionResult GridViewListViewInit();
        #endregion

        #region actionresults
        /// <summary>
        /// GET: /ControllerName/Index.
        /// </summary>
        /// <returns>
        /// Index view.
        /// </returns>
        public virtual ActionResult Index()
        {
            return null;
        }

        /// <summary>
        /// Delete users gridview layout from database.
        /// </summary>
        /// <returns>
        /// The <see cref="ActionResult"/>.
        /// </returns>
        [AjaxOrChildActionOnly]
        public ActionResult ResetGridviewLayout()
        {
            return null;
        }
        #endregion

        #region methods
        /// <summary>
        /// Provide viewdata to identify the gridview as an admin gridview
        /// if this is set, the edit and delete-option for gridview entries is enabled.
        /// </summary>
        public void SetAsAdminGridView()
        {
        }

        /// <summary>
        /// Provide viewdata to identify gridview as an superuser.
        /// </summary>
        public void SetAsSuperUserGridView()
        {
        }

        /// <summary>
        /// Sorts an given currentRow up or down.
        /// </summary>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="mode">
        /// Up or down.
        /// </param>
        /// <param name="groupId">
        /// Group retialerCompanyId (needed if you want to sort in a specific group in a table).
        /// </param>
        public void SortRow(Guid id, string mode, Guid? groupId = null)
        {
        }
        #endregion

        #region admin edit, add, delete

        // delegate for a stored procedure which handles "delete"
        /// <summary>
        /// The delegate delete.
        /// </summary>
        public delegate int DelegateDelete();

        // delegate for a stored procedure which handles "add"
        /// <summary>
        /// The delegate add.
        /// </summary>
        public delegate int DelegateAdd();

        // delegate for a stored procedure which handles "Edit"
        /// <summary>
        /// The delegate edit.
        /// </summary>
        public delegate int DelegateEdit();

        /// <summary>
        /// Validates the model object for the add form.
        /// </summary>
        /// <param name="model">
        /// Model object.
        /// </param>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="adminController">
        /// </param>
        /// <param name="addAndEditProperties">
        /// </param>
        public void ValidateModelForAddForm(object model, Guid id, IDataManipulator adminController, object addAndEditProperties = null)
        {

        }

        /// <summary>
        /// Call stored procedure, for adding an db dataset, which is given in the delegate param.
        /// </summary>
        /// <param name="addFunction">
        /// </param>
        /// <param name="adminController">
        /// The admin Controller.
        /// </param>
        /// <param name="addAndEditProperties">
        /// </param>
        /// <returns>
        /// Return true if model is valid, otherwise return false. if an error occured return true as well.
        /// </returns>
        public bool DoAdd(DelegateAdd addFunction, IDataManipulator adminController, object addAndEditProperties = null)
        {
            return false;
        }

        /// <summary>
        /// Validates the model object for the edit form.
        /// </summary>
        /// <param name="model">
        /// Model object.
        /// </param>
        /// <param name="id">
        /// The id.
        /// </param>
        /// <param name="adminController">
        /// </param>
        /// <param name="addAndEditProperties">
        /// </param>
        public void ValidateModelForEditForm(object model, Guid id, IDataManipulator adminController, object addAndEditProperties = null)
        {
        }

        /// <summary>
        /// Call stored procedure, for updating an db dataset, which is given in the delegate param.
        /// </summary>
        /// <param name="editFunction">
        /// Delegate function for calling an edit-SP.
        /// </param>
        /// <param name="adminController">
        /// </param>
        /// <param name="addAndEditProperties">
        /// </param>
        /// <returns>
        /// Return true if model is valid, otherwise return false. if an error occured return true as well.
        /// </returns>
        public bool DoEdit(DelegateEdit editFunction, IDataManipulator adminController, object addAndEditProperties = null)
        {
            return false;
        }

        /// <summary>
        /// Deletes a record from the db.
        /// </summary>
        /// <param name="deleteFunction">
        /// The delete function.
        /// </param>
        /// <param name="suppressClientErrorMessage">
        /// Apparently only used for testing.
        /// </param>
        /// <param name="suppressClientInfoMessage">
        /// Necessary when deleting records in popops - the info message would show up in the wrong place otherwise.
        /// </param>
        /// <returns>
        /// The <see cref="bool"/>.
        /// </returns>
        public bool DoDelete(DelegateDelete deleteFunction, bool suppressClientErrorMessage = false, bool suppressClientInfoMessage = false)
        {
            return false;
        }
        #endregion
    }
}